import { Action } from 'popmotion';
import { ComponentType } from 'react';
import { CSSProperties } from 'react';
import { DetailedHTMLFactory } from 'react';
import { FrameData } from 'framesync';
import * as React from 'react';
import { ReactHTML } from 'react';
import { ReactNode } from 'react';
import { ReactSVG } from 'react';
import { Ref } from 'react';
import { RefObject } from 'react';
import { SVGAttributes } from 'react';

declare type ActionFactory = (actionConfig: PopmotionTransitionProps) => Action;

declare class AnimationControls<P = {}> {
    private props;
    private values;
    private ref;
    private variants;
    private baseTarget;
    private overrides;
    private resolvedOverrides;
    private defaultTransition?;
    private children?;
    private isAnimating;
    constructor(values: MotionValuesMap, ref: RefObject<Element>);
    setProps(props: P): void;
    setVariants(variants?: Variants): void;
    setDefaultTransition(transition?: Transition): void;
    setValues(target: Target, isActive?: Set<string>): void;
    checkForNewValues(target: Target): void;
    resolveVariant(variant?: Variant): {
        target?: Target;
        transition?: Transition;
        transitionEnd?: Target;
    };
    getHighestPriority(): number;
    clearOverride(overrideIndex: number): void;
    apply(definition: VariantLabels | TargetAndTransition): void;
    private applyVariantLabels;
    start(definition: AnimationDefinition, opts?: AnimationOptions): Promise<any>;
    setPriorityAnimation(definition: AnimationDefinition, priority: number): void;
    isHighestPriority(priority: number): boolean;
    private animate;
    private animateVariantLabels;
    private animateVariant;
    private animateChildren;
    private checkOverrideIsAnimating;
    private resetIsAnimating;
    stop(): void;
    addChild(controls: AnimationControls): void;
    removeChild(controls: AnimationControls): void;
    resetChildren(): void;
}

declare type AnimationDefinition = VariantLabels | TargetAndTransition | TargetResolver;

declare class AnimationManager {
    private hasMounted;
    private defaultTransition;
    private pendingAnimations;
    private componentControls;
    private variants;
    setVariants(variants: Variants): void;
    setDefaultTransition(transition: Transition): void;
    subscribe(controls: AnimationControls): () => boolean;
    start(definition: Variant | string): Promise<any>;
    stop(): void;
    mount(): void;
    unmount(): void;
}

declare type AnimationOptions = {
    delay?: number;
    priority?: number;
};

declare type Config<T> = {
    transformer?: Transformer<T>;
    parent?: MotionValue<T>;
};

declare type Constraints = {
    left?: number;
    right?: number;
    top?: number;
    bottom?: number;
};

declare interface ControlsProp {
    controls?: AnimationControls;
}

declare const createMotionComponent: <P extends {}>(Component: string | React.ComponentClass<P, any> | React.FunctionComponent<P>) => React.ForwardRefExoticComponent<Pick<P & MotionProps, string | number | Exclude<keyof P, "ref">> & React.RefAttributes<Element>>;

declare type CSSPropertiesWithoutTransition = Omit<CSSProperties, "transition">;

declare type CubicBezier = [number, number, number, number];

declare type CustomMotionComponent = {
    custom: typeof createMotionComponent;
};

declare type Decay = {
    type: "decay";
    modifyTarget?: (v: number) => number;
    power?: number;
    timeConstant?: number;
    restDelta?: number;
    from?: number | string;
    velocity?: number;
    delay?: number;
};

declare type DragDirection = "x" | "y";

declare interface DraggableProps {
    /**
     * Enable dragging for this element
     *
     * Set "x" or "y" to only drag in a specific direction
     * Set "lockDirection" to lock dragging into the initial direction
     *  @default false
     */
    dragEnabled?: boolean | DragDirection | "lockDirection";
    /**
     * Disable global drag locking
     * When using nested dragging, setting this to true will enable parents to also drag
     * @default false
     */
    dragPropagation?: boolean;
    /**
     * Apply constraints to dragging
     * @default false
     */
    dragConstraints?: Constraints;
    /**
     * Allow "dragElasticging" beyond the drag constraints
     * @default false
     */
    dragElastic?: Overdrag;
    /**
     * Allow smooth scrolling
     * @default false
     */
    dragMomentum?: boolean;
    onDragStart?: (e: MouseEvent | TouchEvent) => void;
    onDragEnd?: (e: MouseEvent | TouchEvent) => void;
    onDirectionLock?: (axis: string) => void;
}

declare type Easing = CubicBezier | "linear" | "easeIn" | "easeOut" | "easeInOut" | "circIn" | "circOut" | "circInOut" | "backIn" | "backOut" | "backInOut" | "anticipate" | EasingFunction;

declare type EasingFunction = (v: number) => number;

declare type EventHandler = (event: Event, info: EventInfo) => void;

declare interface EventInfo {
    point: Point;
}

declare type GestureHandlers = PanHandlers & TapHandlers & HoverHandlers;

declare type HoverHandler = (event: Event) => void;

declare interface HoverHandlers {
    hover?: string | TargetAndTransition;
    onHoverStart?: HoverHandler;
    onHoverEnd?: HoverHandler;
}

declare type HTMLElements = UnionStringArray<typeof htmlElements>;

declare const htmlElements: ("object" | "track" | "progress" | "a" | "abbr" | "address" | "area" | "article" | "aside" | "audio" | "b" | "base" | "bdo" | "blockquote" | "body" | "br" | "button" | "canvas" | "caption" | "cite" | "code" | "col" | "colgroup" | "data" | "datalist" | "dd" | "del" | "details" | "dfn" | "dialog" | "div" | "dl" | "dt" | "em" | "embed" | "fieldset" | "figcaption" | "figure" | "footer" | "form" | "h1" | "h2" | "h3" | "h4" | "h5" | "h6" | "head" | "header" | "hgroup" | "hr" | "html" | "i" | "iframe" | "img" | "input" | "ins" | "kbd" | "label" | "legend" | "li" | "link" | "map" | "mark" | "menu" | "meta" | "meter" | "nav" | "noscript" | "ol" | "optgroup" | "option" | "output" | "p" | "param" | "picture" | "pre" | "q" | "rt" | "ruby" | "s" | "samp" | "script" | "section" | "select" | "small" | "source" | "span" | "strong" | "style" | "sub" | "sup" | "table" | "tbody" | "td" | "textarea" | "tfoot" | "th" | "thead" | "time" | "title" | "tr" | "u" | "ul" | "var" | "video" | "wbr" | "menuitem" | "bdi" | "big" | "keygen" | "main" | "rp" | "summary" | "webview")[];

declare type HTMLMotionComponents = {
    [K in HTMLElements]: ComponentType<Omit_2<UnwrapFactory<ReactHTML[K]>, "style"> & MotionProps>;
};

declare type Inertia = {
    type: "inertia";
    modifyTarget?: (v: number) => number;
    bounceStiffness?: number;
    bounceDamping?: number;
    power?: number;
    timeConstant?: number;
    restDelta?: number;
    min?: number;
    max?: number;
    from?: number | string;
    velocity?: number;
    delay?: number;
};

declare type Just = {
    type: "just";
    from?: number | string;
    delay?: number;
    velocity?: number;
};

declare type Keyframes = {
    type: "keyframes";
    values: number[] | string[];
    easings?: Easing[];
    easeAll?: Easing;
    elapsed?: number;
    duration?: number;
    loop?: number;
    flip?: number;
    yoyo?: number;
    from?: number | string;
    to?: number | string;
    velocity?: number;
    delay?: number;
};

declare type MakeMotion<T> = {
    [K in keyof T]: T[K] | MotionValue<T[K]>;
} & MotionStyleProperties;

/**
 * @public
 * @internalremarks At some point we might want to explore a "headless" component that renders only `children` in the event
 */
export declare const motion: MotionComponents;

declare type MotionComponents = CustomMotionComponent & HTMLMotionComponents & SVGMotionComponents;

/**
 * @internal
 */
export declare const MotionPluginContext: React.Context<Partial<MotionPlugins>>;

declare interface MotionPluginProps extends MotionPlugins {
    children?: ReactNode;
}

export declare interface MotionPlugins {
    transformPagePoint: (point: Point) => Point;
}

/**
 *
 * @internal
 * @internalremarks For now I think this should remain a private API for our own use
 * until we can figure out a nicer way of allowing people to add these
 */
export declare const MotionPlugins: ({ children, ...props }: MotionPluginProps) => JSX.Element;

declare interface MotionProps extends GestureHandlers, DraggableProps {
    ref?: Ref<Element>;
    style?: MotionStyle;
    animate?: AnimationManager | TargetAndTransition | VariantLabels;
    initial?: Target | VariantLabels;
    variants?: Variants;
    transition?: Transition;
    inherit?: boolean;
    onUpdate?: OnUpdate;
    render?: boolean;
    onAnimationComplete?: () => void;
    [key: string]: any;
}

declare type MotionStyle = MakeMotion<CSSProperties>;

declare type MotionStyleProp = string | number | MotionValue;

declare interface MotionStyleProperties {
    x?: MotionStyleProp;
    y?: MotionStyleProp;
    z?: MotionStyleProp;
    rotate?: MotionStyleProp;
    rotateX?: MotionStyleProp;
    rotateY?: MotionStyleProp;
    rotateZ?: MotionStyleProp;
    scale?: MotionStyleProp;
    scaleX?: MotionStyleProp;
    scaleY?: MotionStyleProp;
    scaleZ?: MotionStyleProp;
    skew?: MotionStyleProp;
    skewX?: MotionStyleProp;
    skewY?: MotionStyleProp;
    originX?: MotionStyleProp;
    originY?: MotionStyleProp;
    originZ?: MotionStyleProp;
    perspective?: MotionStyleProp;
}

/**
 * @public
 */
export declare class MotionValue<V = any> {
    private current;
    private prev;
    private timeDelta;
    private lastUpdated;
    private children?;
    private parent?;
    private updateSubscribers?;
    private renderSubscribers?;
    private transformer?;
    private controller?;
    private canTrackVelocity;
    constructor(init: V, { transformer, parent }?: Config<V>);
    addChild(config: Config<V>): MotionValue<V>;
    removeChild(child: MotionValue): void;
    subscribeTo(subscriptions: Set<Subscriber<V>>, subscription: Subscriber<V>): () => boolean;
    addUpdateSubscription(subscription: Subscriber<V>): () => boolean;
    addRenderSubscription(subscription: Subscriber<V>): () => boolean;
    set(v: V, render?: boolean): void;
    notifySubscriber: (subscriber: Subscriber<V>) => void;
    scheduleVelocityCheck: () => import("framesync/lib/types").Process;
    velocityCheck: ({ timestamp }: FrameData) => void;
    setChild: (child: MotionValue<any>) => void;
    get(): V;
    getVelocity(): number;
    control(controller: ActionFactory, config: PopmotionTransitionProps, transformer?: Transformer<V>): Promise<{}>;
    stop(): void;
    destroy(): void;
}

declare class MotionValuesMap {
    private hasMounted;
    private styler;
    private onUpdate?;
    private shouldRender;
    private values;
    private unsubscribers;
    has(key: string): boolean;
    set(key: string, value: MotionValue): void;
    get<Value>(key: string): MotionValue<Value> | undefined;
    get<Value>(key: string, defaultValue: Value): MotionValue<Value>;
    forEach(callback: (value: MotionValue, key: string) => void): void;
    bindValueToStyler(key: string, value: MotionValue): void;
    setOnUpdate(onUpdate?: OnUpdate): void;
    setShouldRender(render: boolean): void;
    mount(element: Element): void;
    unmount(): void;
}

declare type None = {
    type: false;
    from?: number | string;
    delay?: number;
    velocity?: number;
};

declare type Omit<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;

declare type Omit_2<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;

declare type OnUpdate = (v: Target) => void;

declare type Overdrag = boolean | number;

declare type PanHandler = (event: Event, info: PanInfo) => void;

declare interface PanHandlers {
    onPan?: PanHandler;
    onPanStart?: PanHandler;
    onPanEnd?: PanHandler;
}

declare interface PanInfo {
    point: Point;
    delta: Point;
    offset: Point;
    velocity: Point;
}

declare type Physics = {
    type: "physics";
    acceleration?: number;
    friction?: number;
    restSpeed?: number | false;
    from?: number | string;
    velocity?: number;
    delay?: number;
};

declare interface Point {
    x: number;
    y: number;
}

declare namespace Point {
    /** @alpha */
    const subtract: (a: Point, b: Point) => Point;
}

declare type PopmotionTransitionProps = Tween | Spring | Decay | Keyframes | Physics | Inertia | Just;

declare type Spring = {
    type: "spring";
    stiffness?: number;
    damping?: number;
    mass?: number;
    restSpeed?: number;
    restDelta?: number;
    from?: number | string;
    to?: number | string;
    velocity?: number;
    delay?: number;
};

declare type Subscriber<T> = (v: T) => void;

declare type SVGElements = UnionStringArray<typeof svgElements>;

declare const svgElements: (keyof ReactSVG)[];

declare type SVGMotionComponents = {
    [K in SVGElements]: ComponentType<Omit_2<SVGAttributes<SVGElement>, "style"> & MotionProps>;
};

declare type TapHandler = (session: TapInfo, event: Event) => void;

declare interface TapHandlers {
    onTap?: TapHandler;
    onTapStart?: TapHandler;
    onTapCancel?: TapHandler;
    tap?: string | TargetAndTransition;
}

declare interface TapInfo {
    point: Point;
}

declare type Target = CSSPropertiesWithoutTransition & {
    x?: number | string;
    y?: number | string;
    z?: number | string;
    rotate?: number | string;
    rotateX?: number | string;
    rotateY?: number | string;
    rotateZ?: number | string;
    scale?: number | string;
    scaleX?: number | string;
    scaleY?: number | string;
    scaleZ?: number | string;
    skew?: number | string;
    skewX?: number | string;
    skewY?: number | string;
    originX?: number | string;
    originY?: number | string;
    originZ?: number | string;
    pathLength?: number;
    pathSpacing?: number;
};

declare type TargetAndTransition = Target & {
    transition?: Transition;
    transitionEnd?: Target;
};

declare type TargetResolver = (props: any, current: Target, velocity: Target) => TargetAndTransition;

declare type Transformer<T> = (v: T) => T;

declare type Transformer_2 = (v: any) => any;

declare type TransformerOptions = {
    clamp: true;
};

declare type Transition = (TransitionOrchestration & TransitionDefinition) | (TransitionOrchestration & TransitionMap);

declare type TransitionDefinition = Tween | Spring | Decay | Keyframes | Physics | Inertia | Just | None;

declare type TransitionMap = TransitionOrchestration & {
    [key: string]: TransitionDefinition;
};

declare type TransitionOrchestration = {
    delay?: number;
    beforeChildren?: boolean;
    afterChildren?: boolean;
    delayChildren?: number;
    staggerChildren?: number;
    staggerDirection?: 1 | -1;
};

declare type Tween = {
    type?: "tween";
    duration?: number;
    ease?: Easing;
    elapsed?: number;
    loop?: number;
    flip?: number;
    yoyo?: number;
    from?: number | string;
    to?: number | string;
    velocity?: number;
    delay?: number;
};

declare type UnionStringArray<T extends string[]> = T[number];

declare type UnwrapFactory<F> = F extends DetailedHTMLFactory<infer P, any> ? P : never;

/**
 * If the provided value is a MotionValue, this returns the actual value, otherwise just the value itself
 * @param value
 * @public
 */
export declare function unwrapMotionValue<V>(value: V | MotionValue<V>): V;

/**
 *
 * @param variants
 * @param defaultTransition
 * @public
 */
export declare const useAnimation: (variants?: Variants | undefined, defaultTransition?: import("../types").TransitionMap | (import("../types").TransitionOrchestration & import("../types").Tween) | (import("../types").TransitionOrchestration & import("../types").Spring) | (import("../types").TransitionOrchestration & import("../types").Decay) | (import("../types").TransitionOrchestration & import("../types").Inertia) | (import("../types").TransitionOrchestration & import("../types").Keyframes) | (import("../types").TransitionOrchestration & import("../types").Physics) | (import("../types").TransitionOrchestration & import("../types").Just) | (import("../types").TransitionOrchestration & import("../types").None) | undefined) => AnimationManager;

/**
 * Cycles through a series of states.
 *
 * @remarks
 * `useCycle` works similar to React's `useState`.
 *
 * It's provided an array of possible states, and returns an array of two arguments.
 *
 * The first item is the current state.
 * The second item is a function that cycles to the next state.
 *
 * ```jsx
 * const [x, cycleX] = useCycle([0, 100, 200])
 *
 * return <Frame animate={{ x }} onClick={cycleX} />
 * ```
 *
 * By default, the initial state is the first item in the provided array (`0` in the above example).
 *
 * `useCycle` accepts a second parameter, that can set a different index. For example:
 *
 * ```jsx
 * const [x, cycleX] = useCycle([0, 100, 200], 2)
 * ```
 *
 * `x` would initially be `200`.
 *
 * The returned `cycleState` function, by default, cycles to the next item in the array.
 *
 * Optionally, it accepts an index that will cycle to a specific item in the array.
 *
 * @param items - An array of the possible states
 * @param initialIndex - Index of initial state. Optional, defaults to 0
 * @returns [currentState, cycleState]
 *
 * @public
 */
export declare const useCycle: <T>(items: T[], initialIndex?: number) => [T, (i?: any) => void];

/**
 *
 * @param props
 * @param ref
 * @public
 */
export declare const useGestures: <P extends GestureHandlers>(props: P, ref: RefObject<Element>) => void;

/**
 * For advanced use-cases, you can assume external control of the motion values used by `motion` components.
 *
 * This is usually used in conjunction with `useTransform`.
 *
 * ## Import
 *
 * ```javascript
 * import { useMotionValue } from 'framer-motion'
 * ```
 *
 * ## Usage
 *
 * Motion values are created with the `useMotionValue` hook, providing it an initial value:
 *
 * ```javascript
 * const x = useMotionValue(0)
 * ```
 *
 * This can be passed to a motion component via the `motionValue` prop:
 *
 * ```javascript
 * const MotionComponent = motion.div()
 *
 * export const () => {
 *   const x = useMotionValue(0)
 *
 *   return <MotionComponent motionValues={{ x }} />
 * }
 * ```
 *
 * @param {number | string} init - The initial state of the `MotionValue`
 * @public
 */
export declare const useMotionValue: <T>(init: T) => MotionValue<T>;

/**
 *
 * @param handlers
 * @param ref
 * @public
 */
export declare function usePanGesture(handlers: PanHandlers, ref: RefObject<Element>): undefined;

export declare function usePanGesture(handlers: PanHandlers): {
    onPointerDown: EventHandler;
};

/**
 *
 * @param handlers
 * @public
 */
export declare function useTapGesture(handlers: TapHandlers & ControlsProp): {
    onPointerDown: EventHandler;
};

export declare function useTapGesture(handlers: TapHandlers & ControlsProp, ref: RefObject<Element>): undefined;

/**
 *
 * @param value
 * @param transform
 * @public
 */
export declare function useTransformedValue(value: MotionValue, transform: Transformer_2): MotionValue;

export declare function useTransformedValue(value: MotionValue<number>, from: number[], to: any[], options?: TransformerOptions): MotionValue;

/**
 * `useViewportScroll` provides `MotionValue`s that update when the viewport scrolls.
 *
 * This makes it possible to transform viewport scroll into other values.
 *
 * For instance, highlighting different table of contents items to correspond with page scroll.
 * @public
 */
export declare const useViewportScrollValues: () => {
    scrollX: import(".").MotionValue<number>;
    scrollY: import(".").MotionValue<number>;
    scrollXProgress: import(".").MotionValue<number>;
    scrollYProgress: import(".").MotionValue<number>;
};

declare type Variant = TargetAndTransition | TargetResolver;

declare type VariantLabels = string | string[];

declare type Variants = {
    [key: string]: Variant;
};

export { }
